@@ -35,6 +35,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "database/database.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
+#include "cif/cif.h"
+#include "calma/calma.h"
 #include "utils/utils.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -481,6 +483,23 @@ CIFParseStart()
     return TRUE;
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ * checkFaultFunc ---
+ *
+ *	Callback function for checking if any paint has been generated
+ *	on the CIF target plane for a "fault" layer.
+ * ----------------------------------------------------------------------------
+ */
+
+int checkFaultFunc(tile, clientData)
+    Tile *tile;
+    ClientData clientData;
+{
+    return 1;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -491,7 +510,8 @@ CIFParseStart()
  *	CIF cell.
  *
  * Results:
- *	None.
+ *	Return 0 on normal processing, 1 if any "fault" layer
+ *	returned any geometry.
  *
  * Side effects:
  *	Lots of information gets added to the current Magic cell.
@@ -499,17 +519,39 @@ CIFParseStart()
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 CIFPaintCurrent()
 {
     Plane *plane, *swapplane;
     int i;
+    int checkFault();
 
-    for (i = 0; i < cifCurReadStyle->crs_nLayers; i += 1)
+    for (i = 0; i < cifCurReadStyle->crs_nLayers; i++)
+    {
+	if (cifCurReadStyle->crs_layers[i]->crl_flags & CIFR_FAULTLAYER)
+	{
+	    plane = CIFGenLayer(cifCurReadStyle->crs_layers[i]->crl_ops,
+			&TiPlaneRect, (CellDef *) NULL, cifCurReadPlanes);
+	
+	    if (DBSrPaintArea((Tile *)NULL, plane, &TiPlaneRect,
+			&CIFSolidBits, checkFaultFunc, (ClientData)NULL))
+	    {
+		/* Clean up before returning */
+		DBFreePaintPlane(plane);
+		TiFreePlane(plane);
+		return 1;
+	    }
+	}
+    }
+
+    for (i = 0; i < cifCurReadStyle->crs_nLayers; i++)
     {
 	TileType type;
 	extern int cifPaintCurrentFunc();	/* Forward declaration. */
 
+	if (cifCurReadStyle->crs_layers[i]->crl_flags & CIFR_FAULTLAYER)
+	    continue;
+
 	plane = CIFGenLayer(cifCurReadStyle->crs_layers[i]->crl_ops,
 	    &TiPlaneRect, (CellDef *) NULL, cifCurReadPlanes);
 	
@@ -542,6 +584,8 @@ CIFPaintCurrent()
 
     for (i = 0; i < MAXCIFRLAYERS; i++)
 	DBClearPaintPlane(cifCurReadPlanes[i]);
+
+    return 0;
 }
 
 /* Below is the search function invoked for each CIF tile type
@@ -1152,7 +1196,8 @@ CIFParseUser()
  */
 
 void
-CIFReadCellCleanup()
+CIFReadCellCleanup(type)
+    int type;		 // 0 = CIF, 1 = GDS, because routine is used by both
 {
     HashEntry *h;
     HashSearch hs;
@@ -1161,7 +1206,10 @@ CIFReadCellCleanup()
 
     if (cifSubcellBeingRead)
     {
-	CIFReadError("CIF ended partway through a symbol definition.\n");
+	if (type == 0)
+	    CIFReadError("CIF ended partway through a symbol definition.\n");
+	else
+	    calmaReadError("GDS ended partway through a symbol definition.\n");
 	(void) CIFParseFinish();
     }
 
@@ -1174,12 +1222,18 @@ CIFReadCellCleanup()
 	def = (CellDef *) HashGetValue(h);
 	if (def == NULL)
 	{
-	    CIFReadError("cell table has NULL entry (Magic error).\n");
+	    if (type == 0)
+		CIFReadError("cell table has NULL entry (Magic error).\n");
+	    else
+		calmaReadError("cell table has NULL entry (Magic error).\n");
 	    continue;
 	}
 	if (!(def->cd_flags & CDAVAILABLE))
 	{
-	    CIFReadError("cell %s was used but not defined.\n", def->cd_name);
+	    if (type == 0)
+		CIFReadError("cell %s was used but not defined.\n", def->cd_name);
+	    else
+		calmaReadError("cell %s was used but not defined.\n", def->cd_name);
         }
 	if (def->cd_flags & CDPROCESSEDGDS)
 	{
@@ -1207,20 +1261,38 @@ CIFReadCellCleanup()
 	    if (def->cd_parents != (CellUse *)NULL)
 	    {
 		CellUse *cu_p = def->cd_parents;
-		TxError("GDS read warning:  Cell %s has parent %s\n", savename,
-				cu_p->cu_id);
+		if (type == 0)
+		    CIFReadError("CIF read warning:  Cell %s has parent %s\n",
+				savename, cu_p->cu_id);
+		else
+		    calmaReadError("GDS read warning:  Cell %s has parent %s\n",
+				savename, cu_p->cu_id);
 		def->cd_parents = (CellUse *)NULL;
 	    }
 	    if (DBCellDeleteDef(def) == FALSE)
-		TxError("GDS read error:  Unable to delete cell %s\n", savename);
+	    {
+		if (type == 0)
+		    CIFReadError("CIF read error:  Unable to delete cell %s\n",
+				savename);
+		else
+		    calmaReadError("GDS read error:  Unable to delete cell %s\n",
+				savename);
+	    }
 	    else
-		TxPrintf("GDS read:  Removed flattened cell %s\n", savename);
+	    {
+		if (type == 0)
+		    TxPrintf("CIF read:  Removed flattened cell %s\n", savename);
+		else
+		    TxPrintf("GDS read:  Removed flattened cell %s\n", savename);
+	    }
 	    UndoEnable();
 	    freeMagic(savename);
 	}
 	else
 	{
-	    DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
+	    if ((type == 0 && CIFNoDRCCheck == FALSE) ||
+			(type == 1 && CalmaNoDRCCheck == FALSE))
+		DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
 	    DBWAreaChanged(def, &def->cd_bbox, DBW_ALLWINDOWS, &DBAllButSpaceBits);
 	    DBCellSetModified(def, TRUE);
 	}
