@@ -62,7 +62,51 @@ CIFOp *cifCurReadOp;			/* Last geometric operation seen. */
 void cifReadStyleInit();
 void CIFReadLoadStyle();
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFReadTechLimitScale --
+ *
+ *	Determine if the scalefactor (ns / ds), applied to the current
+ *	grid scaling, would result in a grid finer than the minimum
+ *	resolution allowed by the process, as set by the "gridlimit"
+ *	statement in the "cifinput" section.
+ *
+ *	Note that even if the scalefactor is larger than the minimum
+ *	grid, it must be a MULTIPLE of the minimum grid, or else geometry
+ *	can be generated off-grid.
+ *
+ * Results:
+ *	TRUE if scaling by (ns / ds) would violate minimum grid resolution,
+ *	FALSE if not.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+CIFReadTechLimitScale(ns, ds)
+    int ns, ds;
+{
+    int gridup, scaledown;
+    int scale, limit, mult;
+
+    limit = cifCurReadStyle->crs_gridLimit;
+    if (limit == 0) return FALSE;	/* No limit */
+
+    scale = cifCurReadStyle->crs_scaleFactor;
+    mult = cifCurReadStyle->crs_multiplier;
+
+    gridup = limit * mult * ds;
+    scaledown = scale * ns * 10;
+
+    if ((scaledown / gridup) == 0) return TRUE;
+    if ((scaledown % gridup) != 0) return TRUE;
+    return FALSE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -313,6 +357,8 @@ cifReadStyleInit()
     cifCurReadStyle->crs_cifLayers = DBZeroTypeBits;
     cifCurReadStyle->crs_nLayers = 0;
     cifCurReadStyle->crs_scaleFactor = 0;
+    cifCurReadStyle->crs_multiplier = 1;
+    cifCurReadStyle->crs_gridLimit = 0;
     HashInit(&(cifCurReadStyle->cifCalmaToCif), 64,
       sizeof (CalmaLayerType) / sizeof (unsigned));
     for (i = 0; i < MAXCIFRLAYERS; i++)
@@ -585,6 +631,27 @@ CIFReadTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* Process "gridlimit" lines. */
+    
+    if (strncmp(argv[0], "grid", 4) == 0)
+    {
+        if (StrIsInt(argv[1]))
+        {
+            cifCurReadStyle->crs_gridLimit = atoi(argv[1]);
+            if (cifCurReadStyle->crs_gridLimit < 0)
+            {
+                TechError("Grid limit must be a positive integer.\n");
+                cifCurReadStyle->crs_gridLimit = 0;
+            }
+        }
+        else
+	{
+            TechError("Unable to parse grid limit value.\n");
+	    goto errorReturn;
+	}
+        return TRUE;
+    }
+
     /* Process "variant" lines next. */
 
     if (strncmp(argv[0], "variant", 7) == 0)
@@ -725,6 +792,52 @@ CIFReadTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* Process fault lines next.  (fault handling added 3/16/15) */
+
+    if (strcmp(argv[0], "fault") == 0)
+    {
+	TileType type;
+
+	cifCurReadLayer = NULL;
+	cifCurReadOp = NULL;
+	if (cifCurReadStyle->crs_nLayers == MAXCIFRLAYERS)
+	{
+	    TechError("Can't handle more than %d layers per style.\n",
+		    MAXCIFRLAYERS);
+	    TechError("Your local Magic wizard can increase the table size.\n");
+	    goto errorReturn;
+	}
+        if ((argc != 2) && (argc != 3)) goto wrongNumArgs;
+	type = CIFReadNameToType(argv[1], TRUE);
+	if (type < 0) goto errorReturn;
+
+	cifCurReadLayer = (CIFReadLayer *) mallocMagic(sizeof(CIFReadLayer));
+	cifCurReadStyle->crs_layers[cifCurReadStyle->crs_nLayers]
+		= cifCurReadLayer;
+	cifCurReadStyle->crs_nLayers += 1;
+	cifCurReadLayer->crl_magicType = type;
+	cifCurReadLayer->crl_ops = NULL;
+	cifCurReadLayer->crl_flags = CIFR_FAULTLAYER | CIFR_SIMPLE;
+
+	/* Handle a special case of a list of layer names on the
+	 * layer line.  Turn them into an OR operation.
+	 */
+	
+	if (argc == 3)
+	{
+	    cifCurReadOp = (CIFOp *) mallocMagic(sizeof(CIFOp));
+	    cifCurReadOp->co_opcode = CIFOP_OR;
+	    cifCurReadOp->co_client = (ClientData)NULL;
+	    CIFParseReadLayers(argv[2], &cifCurReadOp->co_cifMask);
+	    TTMaskZero(&cifCurReadOp->co_paintMask);
+	    cifCurReadOp->co_next = NULL;
+	    cifCurReadOp->co_distance = 0;
+	    cifCurReadLayer->crl_ops = cifCurReadOp;
+	}
+	return TRUE;
+    }
+
+
     /* Process mapping between CIF layers and calma layers/types */
     if ((strcmp(argv[0], "calma") == 0) || (strncmp(argv[0], "gds", 3) == 0))
     {
@@ -1225,7 +1338,7 @@ cifParseCalmaNums(str, numArray, numNums)
 /*
  * ----------------------------------------------------------------------------
  *
- * CIFTechInputScale(n, d)
+ * CIFTechInputScale(n, d, opt)
  *
  *   Scale all CIF input scale factors to make them equivalent
  *   to reducing the magic internal unit spacing by a factor of n/d.
