@@ -18,7 +18,7 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFread.h,v 1.2 2009/05/03 15:01:04 tim Exp $
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFread.h,v 1.3 2010/08/25 17:33:55 tim Exp $
  */
 
 #ifndef _CIFREAD_H
@@ -52,10 +52,19 @@ typedef struct
  *		a single OR operation, so it can be handled specially.
  * CIFR_TEMPLAYER: Means this layer is a temporary CIF layer, and that
  *		the "crl_magicType" should be interpreted as a CIF layer.
+ * CIFR_FAULTLAYER: Means that this layer is not a match for any other
+ *		known layer or templayer, and is probably an incomplete
+ *		subset of layers that is not suited to magic's database.
+ *		Special handling can be declared to force all cells
+ *		producing non-empty fault layers to be flattened.
+ * CIFR_TEXTLABELS: Means this layer is used for text-only layers, not to
+ *		be moved or electrically connected to any other layer.
  */
 
 #define CIFR_SIMPLE 1
 #define CIFR_TEMPLAYER 2
+#define CIFR_FAULTLAYER 4
+#define CIFR_TEXTLABELS 8
 
 /* The following structure defines a complete CIF read-in style.
  * The constant MAXCIFRLAYERS must be less than TT_MAXTYPES, and
@@ -93,6 +102,12 @@ typedef struct cifrstyle
 				 * So if crs_multiplier = 10, CIF units
 				 * are in nanometers (millimicrons).
 				 */
+    int crs_gridLimit;		/* Input is considered off-grid if on
+				 * a pitch less than crs_gridLimit CIF
+				 * units, and input will be snapped to
+				 * grid rather than scaling the grid
+				 * to accomodate the data.  0 = no limit.
+				 */
 
     TileType crs_labelLayer[MAXCIFRLAYERS];
 				/* Gives the Magic layer to use for labels
