@@ -290,7 +290,7 @@ DBWloadWindow(window, name, ignoreTech, expand)
     CellDef *newEditDef;
     CellUse *newEditUse;
     void DisplayWindow();
-    int res, newEdit;
+    int res, newEdit, error_val;
     int xadd, yadd;
     Rect loadBox;
     char *rootname;
@@ -324,18 +324,21 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	 * read it from disk.
 	 */
 
-	/* Strip off any ".mag" extension from the name */
-	char *dotptr = strrchr(name, '.');
-
-	if (dotptr != NULL)
-	    if (!strcmp(dotptr, ".mag"))
-		*dotptr = '\0';
+	char *dotptr;
 
 	rootname = strrchr(name, '/');
 	if (rootname == NULL)
 	    rootname = name;
 	else
 	    rootname++;
+
+	/* Strip off any ".mag" extension from the name */
+	dotptr = strrchr(rootname, '.');
+
+	if (dotptr != NULL)
+	    if (!strcmp(dotptr, ".mag"))
+		*dotptr = '\0';
+
 	newEditDef = DBCellLookDef(rootname);
 
 	if ((newEditDef != (CellDef *)NULL) && (newEditDef->cd_file != NULL))
@@ -380,10 +383,43 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	if (newEditDef == (CellDef *) NULL)
 	    newEditDef = DBCellNewDef(rootname, (char *) NULL);
 
-	if (!DBCellRead(newEditDef, name, ignoreTech))
+	if (!DBCellRead(newEditDef, name, ignoreTech, &error_val))
 	{
-	    TxPrintf("Creating new cell\n");
-	    DBCellSetAvail(newEditDef);
+	    if (error_val == ENOENT)
+	    {
+		TxPrintf("Creating new cell\n");
+		DBCellSetAvail(newEditDef);
+	    }
+	    else
+	    {
+		/* File exists but some error has occurred like
+		 * file is unreadable or max file descriptors
+		 * was reached, in which csae we don't want to
+		 * create a new cell, so delete the new celldef
+		 * and return.
+		 */
+		UndoDisable();
+		DBCellDeleteDef(newEditDef);
+		UndoEnable();
+
+		/*
+		 * Go back to loading cell (UNNAMED) if
+		 * there is no EditRootDef or EditCellUse.
+		 * Otherwise, on error, keep whatever was already
+		 * the root def & use.
+		 */
+		if (EditRootDef == NULL || EditCellUse == NULL)
+		{
+		    newEditDef = DBCellLookDef(UNNAMED);
+		    if (newEditDef == (CellDef *) NULL)
+		    {
+			newEditDef = DBCellNewDef(UNNAMED, (char *) NULL);
+			DBCellSetAvail(newEditDef);
+		    }
+		}
+		else
+		    return;
+	    }
 	}
 	else
 	{
