@@ -197,10 +197,12 @@ drcCifWidth(argc, argv)
 
     scalefactor = drcCifStyle->cs_scaleFactor;
     centidistance *= drcCifStyle->cs_expander;		// BSI
+
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcAssign(dpnew, centidistance, dpnext, &CIFSolidBits,
-    		&CIFSolidBits, why, centidistance, DRC_FORWARD, thislayer, 0);
+    		&CIFSolidBits, why, centidistance,
+		DRC_FORWARD, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
@@ -293,16 +295,34 @@ drcCifSpacing(argc, argv)
     drcAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
     		&cmask, why, centidistance, DRC_FORWARD, layer[1], 0);
     drcCifRules[layer[0]][DRC_CIF_SOLID] = dpnew;
+    if (needReverse) dpnew->drcc_flags |= DRC_BOTHCORNERS;
+
+    // Add rule in reverse direction
+    dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
+    dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
+    drcAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
+    		&cmask, why, centidistance, DRC_REVERSE, layer[1], 0);
+    drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
     
     if (needReverse)
     {
+	 // This is not so much "reverse" as it is just the
+	 // rule for b->a spacing that matches the a->b spacing.
+
          dpnew->drcc_flags |= DRC_BOTHCORNERS;
          dpnext = drcCifRules[layer[1]][DRC_CIF_SOLID];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
 		why, centidistance, DRC_FORWARD|DRC_BOTHCORNERS, layer[0], 0);
          drcCifRules[layer[1]][DRC_CIF_SOLID] = dpnew;
-	 
+
+	 // Add rule in reverse direction
+	 dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
+	 dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
+	 drcAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
+		why, centidistance, DRC_REVERSE|DRC_BOTHCORNERS, layer[0], 0);
+	 drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
+    
 	 if (layer[0] == layer[1])
 	 {
               dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
@@ -503,9 +523,9 @@ drcCifCheck(arg)
 	    for (drcCifCur = drcCifRules[i][j]; 
 	       		drcCifCur; drcCifCur = drcCifCur->drcc_next)
             {
-     	        Plane *plane = CIFPlanes[i];
 	  	TileTypeBitMask	*mask;
 	  
+		arg->dCD_plane = i;
 	        DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &cifrect,
 			(j == DRC_CIF_SOLID) ? &DBSpaceBits : &CIFSolidBits,
 	  		drcCifTile, arg);
