@@ -54,7 +54,8 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 
 #ifdef MAGIC_WRAPPER
 char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
-	"devcap", "diode", "subckt", "rsubckt", "msubckt", NULL};
+	"devcap", "devcaprev", "diode", "pdiode", "ndiode", "subckt",
+	"rsubckt", "msubckt", NULL};
 #endif
 
 /* --------------------- Data local to this file ---------------------- */
@@ -110,8 +111,8 @@ typedef struct LB1
 } LinkedBoundary;
 
 LinkedBoundary **extSpecialBounds;	/* Linked Boundary List */
-NodeRegion  	*glob_subsnode;		/* Global substrate node */
-static Label	*subsLabel = NULL;	/* Label used for substrate */
+NodeRegion *glob_subsnode = NULL;	/* Global substrate node */
+NodeRegion *temp_subsnode = NULL;	/* Last subsnode found */
 
 /* Structure used for finding substrate connections on implicitly-defined
  * substrates
@@ -131,7 +132,6 @@ typedef struct TSD1
 void extOutputNodes();
 int extTransTileFunc();
 int extTransPerimFunc();
-NodeRegion *extTransFindSubsNode();
 int extTransFindSubs();
 
 int extAnnularTileFunc();
@@ -211,7 +211,9 @@ extBasic(def, outFile)
      * Use a special-purpose version of ExtFindRegions for speed.
      */
     if (!SigInterruptPending)
-	nodeList = extFindNodes(def, (Rect *) NULL);
+	nodeList = extFindNodes(def, (Rect *) NULL, FALSE);
+
+    glob_subsnode = temp_subsnode;	// Keep a record of the def's substrate
 
     /* Assign the labels to their associated regions */
     if (!SigInterruptPending)
@@ -238,8 +240,10 @@ extBasic(def, outFile)
 	/* Convert coupling capacitance to the substrate node to
 	 * substrate capacitance on each node in nreg_cap
 	 */
-	if (!SigInterruptPending && (ExtOptions&EXT_DOCOUPLING))
-	    extRelocateSubstrateCoupling(&extCoupleHash, glob_subsnode);
+
+	if (ExtCurStyle->exts_globSubstratePlane != -1)
+	    if (!SigInterruptPending && (ExtOptions&EXT_DOCOUPLING))
+		extRelocateSubstrateCoupling(&extCoupleHash, glob_subsnode);
     }
 
     /* Output device parameters for any subcircuit devices */
@@ -416,14 +420,10 @@ extOutputNodes(nodeList, outFile)
 	/* Check if this node is the substrate */
 	if (reg == glob_subsnode)
 	{
-	    if (((LabRegion *)reg)->lreg_labels == NULL)
-		fprintf(outFile, "substrate \"0\" 0 0");
-	    else
-		fprintf(outFile, "substrate \"%s\" 0 0", text);
+	    fprintf(outFile, "substrate \"%s\" 0 0", text);
 	}
 	else
 	{
-
 	    intR = (reg->nreg_resist + rround) / ExtCurStyle->exts_resistScale;
 	    finC = reg->nreg_cap/ExtCurStyle->exts_capScale;
 	    fprintf(outFile, "node \"%s\" %d %lg", text, intR, finC);
@@ -1255,7 +1255,7 @@ extGetNativeResistClass(type, term)
     for (i = 0;; i++)
     {
 	rmask = &ExtCurStyle->exts_transSDTypes[type][i];
-	if (TTMaskHasType(rmask, TT_SPACE)) break;
+	if (TTMaskIsZero(rmask)) break;
 	tmask = rmask;
 	if (i == term) break;
     }
@@ -1270,6 +1270,78 @@ extGetNativeResistClass(type, term)
     return -1; 		/* Error */
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extOutputDevParams ---
+ *
+ *	Write information to the output in the form of parameters
+ *	representing pre-defined aspects of the device geometry
+ *	that may be specified for any device.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Writes non-terminated output to the file 'outFile'.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+extOutputDevParams(reg, t, outFile, length, width)
+    TransRegion *reg;
+    TileType t;
+    FILE *outFile;
+    int length;
+    int width;
+{
+    ParamList *chkParam;
+
+    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
+		!= NULL; chkParam = chkParam->pl_next)
+    {
+	switch(tolower(chkParam->pl_param[0]))
+	{
+	    case 'a':
+		if (chkParam->pl_param[1] == '\0' ||
+			chkParam->pl_param[1] == '0')
+		    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+				reg->treg_area);
+		break;
+	    case 'p':
+		if (chkParam->pl_param[1] == '\0' ||
+			chkParam->pl_param[1] == '0')
+		    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+				extTransRec.tr_perim);
+		break;
+	    case 'l':
+		fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+				length);
+		break;
+	    case 'w':
+		fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+				width);
+		break;
+	    case 'c':
+		fprintf(outFile, " %c=%g", chkParam->pl_param[0],
+			(ExtCurStyle->exts_transGateCap[t]
+			* reg->treg_area) +
+			(ExtCurStyle->exts_transSDCap[t]
+			* extTransRec.tr_perim));
+		break;
+	    case 's':
+	    case 'x':
+	    case 'y':
+		/* Do nothing;  these values are standard output */
+		break;
+	    default:
+		fprintf(outFile, " %c=", chkParam->pl_param[0]);
+		break;
+	}
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1317,7 +1389,6 @@ extOutputTrans(def, transList, outFile)
     double dres, dcap;
     char mesg[256];
     bool isAnnular, hasModel;
-    ParamList *chkParam;
 
     for (reg = transList; reg && !SigInterruptPending; reg = reg->treg_next)
     {
@@ -1362,6 +1433,38 @@ extOutputTrans(def, transList, outFile)
 	arg.fra_each = (int (*)()) NULL;
 	(void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
+	/* Are the terminal types on a compeletely different	*/
+	/* plane than the top type?  If so, do an area search	*/
+	/* on that plane in the area under the device node.	*/
+
+	/* Devices may define one terminal per plane, but this	*/
+	/* method cannot handle several different layer types	*/
+	/* on one plane under the device identifier layer	*/
+	/* acting as separate device nodes.  If any terminal	*/
+	/* search fails, give up and proceed with the reduced	*/
+	/* number of terminals.					*/
+
+	while (extTransRec.tr_nterm < nsd)
+	{
+	    TileTypeBitMask *tmask;
+
+	    tmask = &ExtCurStyle->exts_transSDTypes[t][extTransRec.tr_nterm];
+	    if (TTMaskIsZero(tmask)) break;
+	    if (!TTMaskIntersect(tmask, &DBPlaneTypes[reg->treg_pnum]))
+	    {
+		node = NULL;
+		extTransFindSubs(reg->treg_tile, t, tmask, def, &node);
+		if (node == NULL) break;
+		extTransRec.tr_termnode[extTransRec.tr_nterm++] = node;
+	    }
+	    else if (TTMaskHasType(tmask, TT_SPACE)) {
+		/* Device node is specified as being the substrate */
+		if (glob_subsnode == NULL) break;
+		extTransRec.tr_termnode[extTransRec.tr_nterm++] = glob_subsnode;
+	    }
+	    else break;
+	}
+
 	/*
 	 * For types that require a minimum number of terminals,
 	 * check to make sure that they all exist.  If they don't,
@@ -1401,6 +1504,10 @@ extOutputTrans(def, transList, outFile)
 	 * Use the default substrate node unless the transistor overlaps
 	 * material whose type is in exts_transSubstrateTypes, in which
 	 * case we use the node of the overlapped material.
+	 *
+	 * Technology files using the "substrate" keyword (magic-8.1 or
+	 * newer) should have the text "error" in the substrate node
+	 * name.
 	 */
 	subsName = ExtCurStyle->exts_transSubstrateName[t];
 	if (!TTMaskIsZero(&ExtCurStyle->exts_transSubstrateTypes[t])
@@ -1409,6 +1516,23 @@ extOutputTrans(def, transList, outFile)
 	    subsName = extNodeName(subsNode);
 	}
 
+#ifdef MAGIC_WRAPPER
+
+	// Substrate variable substitution when in backwards-compatibility
+	// substrate mode.
+
+	else if ((ExtCurStyle->exts_globSubstratePlane == -1) &&
+		(subsName && subsName[0] == '$' && subsName[1] != '$'))
+	{
+	    // If subsName is a Tcl variable (begins with "$"), make the
+	    // variable substitution, if one exists.  Ignore double-$.
+
+	    char *varsub = (char *)Tcl_GetVar(magicinterp, &subsName[1],
+			TCL_GLOBAL_ONLY);
+	    if (varsub != NULL) subsName = varsub;
+	}
+#endif
+
 	/* Original-style FET record backward compatibility */
 	if (ExtCurStyle->exts_deviceClass[t] != DEV_FET)
 	    fprintf(outFile, "device ");
@@ -1530,63 +1654,23 @@ extOutputTrans(def, transList, outFile)
 
 		}
 
-		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT ||
-			ExtCurStyle->exts_deviceClass[t] == DEV_MSUBCKT)
-		{
-		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
-				!= NULL; chkParam = chkParam->pl_next)
-		    {
-			switch(tolower(chkParam->pl_param[0]))
-			{
-			    case 'a':
-				if (chkParam->pl_param[1] == '\0' ||
-					chkParam->pl_param[1] == '0')
-				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						reg->treg_area);
-				break;
-			    case 'p':
-				if (chkParam->pl_param[1] == '\0' ||
-					chkParam->pl_param[1] == '0')
-				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						extTransRec.tr_perim);
-				break;
-			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						length);
-				break;
-			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						width);
-				break;
-			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
-					(ExtCurStyle->exts_transGateCap[t]
-					* reg->treg_area) +
-					(ExtCurStyle->exts_transSDCap[t]
-					* extTransRec.tr_perim));
-				break;
-			    case 's':
-			    case 'x':
-			    case 'y':
-				/* Do nothing;  these values are standard output */
-				break;
-			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param[0]);
-				break;
-			}
-		    }
-
-		}
-		else  /* DEV_MOSFET, DEV_ASYMMETRIC, DEV_BJT */
+		if (ExtCurStyle->exts_deviceClass[t] == DEV_MOSFET ||
+			ExtCurStyle->exts_deviceClass[t] == DEV_ASYMMETRIC ||
+			ExtCurStyle->exts_deviceClass[t] == DEV_BJT)
 		{
 		    fprintf(outFile, " %d %d", length, width);
 		}
 
+		extOutputDevParams(reg, t, outFile, length, width);
+
 		fprintf(outFile, " \"%s\"", (subsName == NULL) ?
 					"None" : subsName);
 		break;
 
 	    case DEV_DIODE:	/* Only handle the optional substrate node */
+	    case DEV_NDIODE:
+	    case DEV_PDIODE:
+		extOutputDevParams(reg, t, outFile, length, width);
 		if (subsName != NULL)
 		    fprintf(outFile, " \"%s\"", subsName);
 		break;
@@ -1700,54 +1784,10 @@ extOutputTrans(def, transList, outFile)
 					"Resistor has zero width");
 		}
 
+		extOutputDevParams(reg, t, outFile, length, width);
+
 		if (ExtCurStyle->exts_deviceClass[t] == DEV_RSUBCKT)
 		{
-		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
-				!= NULL; chkParam = chkParam->pl_next)
-		    {
-			switch(tolower(chkParam->pl_param[0]))
-			{
-			    case 'a':
-				if (chkParam->pl_param[1] == '\0' ||
-					chkParam->pl_param[1] == '0')
-				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						reg->treg_area);
-				break;
-			    case 'p':
-				if (chkParam->pl_param[1] == '\0' ||
-					chkParam->pl_param[1] == '0')
-				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-						extTransRec.tr_perim);
-				break;
-			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-					length);
-				break;
-			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
-					width);
-				break;
-			    case 'r':
-				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
-					dres / 1000.0);
-				break;
-			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
-					(ExtCurStyle->exts_transGateCap[t]
-					* reg->treg_area) +
-					(ExtCurStyle->exts_transSDCap[t]
-					* extTransRec.tr_perim));
-				break;
-			    case 's':
-			    case 'x':
-			    case 'y':
-				/* Do nothing;  these values are standard output */
-				break;
-			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param[0]);
-				break;
-			}
-		    }
 		    fprintf(outFile, " \"%s\"", (subsName == NULL) ?
 				"None" : subsName);
 		}
@@ -1762,6 +1802,7 @@ extOutputTrans(def, transList, outFile)
 		break;
 
 	    case DEV_CAP:
+	    case DEV_CAPREV:
 		hasModel = strcmp(ExtCurStyle->exts_transName[t], "None");
 		if (hasModel)
 		{
@@ -1779,20 +1820,6 @@ extOutputTrans(def, transList, outFile)
 						"Capacitor has zero size");
 			fprintf(outFile, " 0 0");
 		    }
-//		    else if ((ntiles == 1) || (n > 1))
-//		    {
-//			width = 0;
-//			for (n = 0; extTransRec.tr_termlen[n] != 0; n++)
-//			    width += extTransRec.tr_termlen[n];
-//
-//			if (!width && ExtDoWarn)
-//			    extTransBad(def, reg->treg_tile,
-//					"Capacitor has zero width");
-//			else width >>= 2;	/* total perimeter / 4 */
-//
-//			fprintf(outFile, " %d %d", reg->treg_area / width,
-//				width);
-//		    }
 		    else
 		    {
 			/* Special handling of multiple-tile areas.	*/
@@ -1843,6 +1870,7 @@ extOutputTrans(def, transList, outFile)
 			arg.fra_each = (int (*)()) NULL;
 			(void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 		    }
+		    extOutputDevParams(reg, t, outFile, length, width);
 		    if (subsName != NULL)
 			fprintf(outFile, " \"%s\"", subsName);
 		}
@@ -1866,7 +1894,7 @@ extOutputTrans(def, transList, outFile)
 	/* device is asymmetric, in which case source and drain do not	*/
 	/* permute, and the terminal order is fixed.			*/
 
-	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][1], TT_SPACE))
+	if (TTMaskIsZero(&ExtCurStyle->exts_transSDTypes[t][1]))
 	    ExtSortTerminals(&extTransRec, ll); 
 
 	/* each non-gate terminal */
@@ -1879,23 +1907,18 @@ extOutputTrans(def, transList, outFile)
 }
 
 int
-extTransFindSubs(tile, def, sn)
+extTransFindSubs(tile, t, mask, def, sn)
     Tile *tile;
+    TileType t;
+    TileTypeBitMask *mask;
     CellDef *def;
     NodeRegion **sn;
 {
-    TileType t;
-    TileTypeBitMask *mask;
     Rect tileArea;
     int pNum;
     int extTransFindSubsFunc1();	/* Forward declaration */
 
     TiToRect(tile, &tileArea);
-    if (IsSplit(tile))
-        t = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
-    else
-        t = TiGetTypeExact(tile);
-    mask = &ExtCurStyle->exts_transSubstrateTypes[t];
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
 	if (TTMaskIntersect(&DBPlaneTypes[pNum], mask))
@@ -1956,6 +1979,7 @@ extTransTileFunc(tile, pNum, arg)
     TileTypeBitMask mask;
     TileType loctype;
     int perim;
+    bool allow_globsubsnode;
 
     LabelList *ll;
     Label *lab;
@@ -1996,15 +2020,30 @@ extTransTileFunc(tile, pNum, arg)
 		extTransPerimFunc, (ClientData)NULL);
     extTransRec.tr_perim += perim;
 
+    allow_globsubsnode = FALSE;
     if (extTransRec.tr_subsnode == (NodeRegion *)NULL)
-	extTransFindSubs(tile, arg->fra_def, &extTransRec.tr_subsnode);
+    {
+	TileTypeBitMask *smask;
 
-    /* If the transistor does not connect to a defined node, then it
-     * is assumed to connect to the global substrate.
+	smask = &ExtCurStyle->exts_transSubstrateTypes[loctype];
+	if (TTMaskHasType(smask, TT_SPACE))
+	{
+	    allow_globsubsnode = TRUE;
+	    TTMaskClearType(smask, TT_SPACE);
+	}
+	extTransFindSubs(tile, loctype, smask, arg->fra_def, &extTransRec.tr_subsnode);
+	if (allow_globsubsnode)
+	    TTMaskSetType(smask, TT_SPACE);
+    }
+
+    /* If the transistor does not connect to a defined node, and
+     * the substrate types include "space", then it is assumed to
+     * connect to the global substrate.
      */
 
     if (extTransRec.tr_subsnode == (NodeRegion *)NULL)
-	extTransRec.tr_subsnode = glob_subsnode;
+	if (allow_globsubsnode)
+	    extTransRec.tr_subsnode = glob_subsnode;
 
     return (0);
 }
@@ -2034,9 +2073,15 @@ extTransPerimFunc(bp)
     else
         toutside = TiGetTypeExact(bp->b_outside);
 
-    for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
-		i++)
+    for (i = 0; !TTMaskIsZero(&ExtCurStyle->exts_transSDTypes[tinside][i]); i++)
     {
+	/* TT_SPACE is allowed, for declaring that a device terminal is	*/
+	/* the substrate.  However, it should not be in the plane of	*/
+	/* the device identifier layer, so space tiles should never be	*/
+	/* flagged during a device perimeter search.			*/
+
+	if (toutside == TT_SPACE) break;
+
 	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], toutside))
 	{
 	    /*
@@ -2044,7 +2089,7 @@ extTransPerimFunc(bp)
 	     * already in our table; add it if it wasn't already there.
 	     * Asymmetric devices must have terminals in order.
 	     */
-	    if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][1], TT_SPACE))
+	    if (TTMaskIsZero(&ExtCurStyle->exts_transSDTypes[tinside][1]))
 	    {
 		for (thisterm = 0; thisterm < extTransRec.tr_nterm; thisterm++)
 		    if (extTransRec.tr_termnode[thisterm] == diffNode)
@@ -2293,8 +2338,7 @@ extSpecialPerimFunc(bp, sense)
 
     /* Check all terminal classes for a matching type */
     needSurvey = FALSE;
-    for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
-		i++)
+    for (i = 0; !TTMaskIsZero(&ExtCurStyle->exts_transSDTypes[tinside][i]); i++)
     {
 	if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], toutside))
 	{
@@ -2305,6 +2349,10 @@ extSpecialPerimFunc(bp, sense)
 
     if (!sense || needSurvey)
     {
+	if (toutside == TT_SPACE)
+	    if (glob_subsnode != NULL)
+		diffNode = glob_subsnode;
+
 	/*
 	 * Since we're repeating the search, all terminals should be there.
 	 */
@@ -2317,6 +2365,7 @@ extSpecialPerimFunc(bp, sense)
 		    break;
 	    if (thisterm >= extTransRec.tr_nterm)
 	    {
+		if (toutside == TT_SPACE)
 		TxError("Internal Error in Transistor Perimeter Boundary Search!\n");
 		return 1;
 	    }
@@ -2742,7 +2791,6 @@ extTransEach(tile, pNum, arg)
     return (0);
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2767,11 +2815,12 @@ Stack *extNodeStack = NULL;
 Rect *extNodeClipArea = NULL;
 
 NodeRegion *
-extFindNodes(def, clipArea)
+extFindNodes(def, clipArea, subonly)
     CellDef *def;	/* Def whose nodes are being found */
     Rect *clipArea;	/* If non-NULL, ignore perimeter and area that extend
 			 * outside this rectangle.
 			 */
+    bool subonly;	/* If true, only find the substrate node, and return */
 {
     int extNodeAreaFunc();
     int extSubsFunc();
@@ -2783,9 +2832,6 @@ extFindNodes(def, clipArea)
     for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
 	extResistArea[n] = extResistPerim[n] = 0;
 
-    /* Also reset the substrate node pointer */
-    glob_subsnode = (NodeRegion *)NULL;
-
     extNodeClipArea = clipArea;
     if (extNodeStack == (Stack *) NULL)
 	extNodeStack = StackNew(64);
@@ -2800,6 +2846,8 @@ extFindNodes(def, clipArea)
     /* call extNodeAreaFunc() on the first of these to generate	 */
     /* a single substrate node.					 */
 
+    temp_subsnode = (NodeRegion *)NULL;		// Reset for new search
+
     TTMaskZero(&subsTypesNonSpace);
     TTMaskSetMask(&subsTypesNonSpace, &ExtCurStyle->exts_globSubstrateTypes);
     TTMaskClearType(&subsTypesNonSpace, TT_SPACE);
@@ -2818,45 +2866,32 @@ extFindNodes(def, clipArea)
 
     /* If there was a substrate connection, process it and everything	*/
     /* that was connected to it.  If not, then create a new node	*/
-    /* (named "0" by default) to represent the substrate.		*/
+    /* to represent the substrate.					*/
 
     if (!StackEmpty(extNodeStack))
     {
 	Tile *tile;
 	int tilePlaneNum;
+
 	POPTILE(tile, tilePlaneNum);
 	arg.fra_pNum = tilePlaneNum;
 	extNodeAreaFunc(tile, &arg);
-	glob_subsnode = (NodeRegion *)arg.fra_region;
+	temp_subsnode = (NodeRegion *)arg.fra_region;
     }
-    else
+    else if (ExtCurStyle->exts_globSubstratePlane != -1)
     {
-	LabelList *ll;
+	NodeRegion *loc_subsnode;
 
 	extNodeAreaFunc((Tile *)NULL, (FindRegion *)&arg);
-	glob_subsnode = (NodeRegion *)arg.fra_region;
-	glob_subsnode->nreg_pnum = ExtCurStyle->exts_globSubstratePlane;
-	glob_subsnode->nreg_type = TT_SPACE;
-	glob_subsnode->nreg_ll.p_x = MINFINITY + 3;
-	glob_subsnode->nreg_ll.p_y = MINFINITY + 3;
-
-	if (subsLabel == NULL)
-	{
-	    subsLabel = (Label *) mallocMagic((unsigned) (sizeof (Label)));
-	    subsLabel->lab_type = TT_SPACE;
-	    subsLabel->lab_just = GEO_NORTH;
-	    subsLabel->lab_flags = LABEL_GENERATE;
-	    strcpy(subsLabel->lab_text, "0");
-	    subsLabel->lab_rect = TiPlaneRect;
-	    subsLabel->lab_next = (Label *)NULL;
-	}
-
-	ll = (LabelList *) mallocMagic((unsigned) (sizeof (LabelList)));
-	ll->ll_next = (LabelList *) NULL;
-	ll->ll_label = subsLabel;
-
-	glob_subsnode->nreg_labels = ll;
+	loc_subsnode = (NodeRegion *)arg.fra_region;
+	loc_subsnode->nreg_pnum = ExtCurStyle->exts_globSubstratePlane;
+	loc_subsnode->nreg_type = TT_SPACE;
+	loc_subsnode->nreg_ll.p_x = MINFINITY + 3;
+	loc_subsnode->nreg_ll.p_y = MINFINITY + 3;
+	loc_subsnode->nreg_labels = NULL;
+	temp_subsnode = loc_subsnode;
     }
+    if (subonly == TRUE) return ((NodeRegion *) arg.fra_region);
 
     /* Second pass:  Find all other nodes */
 
