@@ -187,7 +187,7 @@ extSubtree(parentUse, f)
 	    // the labels.
 
 	    for (lab = def->cd_labels; lab; lab = lab->lab_next)
-		if (GEO_OVERLAP(&lab->lab_rect, &r))
+		if (GEO_OVERLAP(&lab->lab_rect, &r) || GEO_TOUCH(&lab->lab_rect, &r))
 		    result |= GeoIncludeAll(&lab->lab_rect, &ha.ha_interArea);
 
 	    if (result)
@@ -355,7 +355,7 @@ extSubtreeInteraction(ha)
 #ifdef	notdef
     extCopyPaint(ha->ha_parentUse->cu_def, &ha->ha_interArea, oneDef);
 #endif	/* notdef */
-    oneFlat->et_nodes = extFindNodes(oneDef, &ha->ha_clipArea);
+    oneFlat->et_nodes = extFindNodes(oneDef, &ha->ha_clipArea, FALSE);
     if ((ExtOptions & (EXT_DOCOUPLING|EXT_DOADJUST))
 		   == (EXT_DOCOUPLING|EXT_DOADJUST))
     {
@@ -392,7 +392,7 @@ extSubtreeInteraction(ha)
 	 * Don't reset ha_lookNames, since we still need to be able to
 	 * refer to nodes in the parent.
 	 */
-	ha->ha_cumFlat.et_nodes = extFindNodes(cumDef, &ha->ha_clipArea);
+	ha->ha_cumFlat.et_nodes = extFindNodes(cumDef, &ha->ha_clipArea, FALSE);
 	ExtLabelRegions(cumDef, ExtCurStyle->exts_nodeConn,
 			&(ha->ha_cumFlat.et_nodes), &ha->ha_clipArea);
 	if (ExtOptions & EXT_DOCOUPLING)
@@ -591,6 +591,7 @@ extSubtreeFunc(scx, ha)
     SearchContext newscx;
     ExtTree *oneFlat;
     HierYank hy;
+    int x, y;
 
     /* Allocate a new ExtTree to hold the flattened, extracted subtree */
     oneFlat = extHierNewOne();
@@ -619,7 +620,7 @@ extSubtreeFunc(scx, ha)
      * hard way.
      */
     oneDef = oneFlat->et_use->cu_def;
-    oneFlat->et_nodes = extFindNodes(oneDef, &ha->ha_clipArea),
+    oneFlat->et_nodes = extFindNodes(oneDef, &ha->ha_clipArea, FALSE);
     ExtLabelRegions(oneDef, ExtCurStyle->exts_nodeConn,
 		&(oneFlat->et_nodes), &ha->ha_clipArea);
     if ((ExtOptions & (EXT_DOCOUPLING|EXT_DOADJUST))
@@ -673,53 +674,39 @@ extSubtreeFunc(scx, ha)
 	 */
 	ha->ha_cumFlat.et_nodes =
 	    (NodeRegion *) ExtFindRegions(cumUse->cu_def, &TiPlaneRect,
-				&DBAllButSpaceBits, ExtCurStyle->exts_nodeConn, extUnInit,
-				extHierLabFirst, (int (*)()) NULL);
+				&DBAllButSpaceBits, ExtCurStyle->exts_nodeConn,
+				extUnInit, extHierLabFirst, (int (*)()) NULL);
 	ExtLabelRegions(cumUse->cu_def, ExtCurStyle->exts_nodeConn,
 			&(ha->ha_cumFlat.et_nodes), &TiPlaneRect);
     }
 
-    /* Process substrate connection 				*/
-    /* save_subsnode connects to glob_subsnode 			*/
-    /* (node names get chained together, one node gets removed)	*/
-
-    if (save_subsnode != NULL && glob_subsnode != NULL &&
-		save_subsnode->nreg_labels != NULL &&
-		glob_subsnode->nreg_labels != NULL)
-    {
-	HashTable *table = &ha->ha_connHash;
-	HashEntry *he;
-	NodeName *nn;
-	Node *node1, *node2;
-
-	/* Register the name, like is done in extHierConnectFunc2 */
-	he = HashFind(table, save_subsnode->nreg_labels->ll_label->lab_text);
-	nn = (NodeName *) HashGetValue(he);
-	node1 = nn ? nn->nn_node : extHierNewNode(he);
-
-	he = HashFind(table, glob_subsnode->nreg_labels->ll_label->lab_text);
-	nn = (NodeName *) HashGetValue(he);
-	node2 = nn ? nn->nn_node : extHierNewNode(he);
-
-	if (node1 != node2)
-	{
-	    /*
-	     * Both sets of names will now point to node1.
-	     * We don't need to update node_cap since it
-	     * hasn't been computed yet.
-	     */
-	    for (nn = node2->node_names; nn->nn_next; nn = nn->nn_next)
-	        nn->nn_node = node1;
-	    nn->nn_node = node1;
-	    nn->nn_next = node1->node_names;
-	    node1->node_names = node2->node_names;
-	    freeMagic((char *) node2);
-	}
-    }
-
     /* Process connections; this updates ha->ha_connHash */
     extHierConnections(ha, &ha->ha_cumFlat, oneFlat);
 
+    /* Process substrate connection.  All substrates should be	*/
+    /* connected together in the cell def, so in the case of an	*/
+    /* array, just make sure that the first array entry is	*/
+    /* connected.						*/
+    
+    if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
+	extHierSubstrate(ha, use, -1, -1);
+    else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
+    {	
+	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
+	    extHierSubstrate(ha, use, -1, y);
+    }
+    else if (use->cu_xhi > use->cu_xlo && use->cu_yhi == use->cu_ylo)
+    {
+	for (x = use->cu_xlo; x <= use->cu_xhi; x++)
+	extHierSubstrate(ha, use, x, -1);
+    }
+    else
+    {
+	for (x = use->cu_xlo; x <= use->cu_xhi; x++)
+	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
+		extHierSubstrate(ha, use, x, y);
+    }
+
     /* Free the cumulative node list we extracted above */
     if (ha->ha_cumFlat.et_nodes)
     {
