@@ -614,12 +614,19 @@ GrTkGetColorByName(name)
 
     if (strlen(name) == 1)
 	style = GrStyleNames[name[0] & 0x7f];
+    else if (DBWNumStyles == 0)
+    {
+	TxError("No style table exists.\n");
+	return NULL;
+    }
     else
     {
 	for (style = 0; style < TECHBEGINSTYLES + DBWNumStyles; style++)
-	    if (!strcmp(name, GrStyleTable[style].longname))
-		break;
+	    if (GrStyleTable[style].longname != NULL)
+		if (!strcmp(name, GrStyleTable[style].longname))
+		    break;
     }
+
     if (style >= TECHBEGINSTYLES + DBWNumStyles)
     {
 	TxError("Style does not exist or style is not accessible\n");
