@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.3 2009/01/30 03:11:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.4 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -27,7 +27,7 @@ CellUse 		*ResUse=NULL;		/* Our use and def */
 CellDef 		*ResDef=NULL;
 TileTypeBitMask 	ResConnectWithSD[NT];	/* A mask that goes from  */
 						/* SD's to transistors.   */
-TileTypeBitMask 	ResCopyMask[NT];	/* Inidicates which tiles */
+TileTypeBitMask 	ResCopyMask[NT];	/* Indicates which tiles */
 						/* are to be copied.      */
 resResistor 		*ResResList=NULL;	/* Resistor list	  */
 resNode     		*ResNodeList=NULL;	/* Processed Nodes 	  */
@@ -281,12 +281,8 @@ ResFindNewContactTiles(contacts)
 		       TxError("Error: setting contact tile to null\n");
 		  }
 #endif
-#ifdef NONMANHATTAN
 		  if ((IsSplit(tile) && TTMaskHasType(&mask, TiGetRightType(tile)))
 			|| TTMaskHasType(&mask, TiGetType(tile)))
-#else
-		  if TTMaskHasType(&mask, TiGetType(tile))
-#endif
 		  {
 		       tileJunk	*j = (tileJunk *)tile->ti_client;
 		       cElement *ce;
@@ -820,7 +816,6 @@ FindStartTile(goodies, SourcePoint)
     tile = ResUse->cu_def->cd_planes[pnum]->pl_hint;
     GOTOPOINT(tile, &workingPoint);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
         if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetLeftType(tile)) != 0)
@@ -840,9 +835,7 @@ FindStartTile(goodies, SourcePoint)
 	    return(NULL);
 	}
     }
-    else
-#endif
-    if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)) == 0)
+    else if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)) == 0)
     {
 	TxError("Couldn't find transistor at %d %d\n",
 		goodies->rg_tranloc->p_x, goodies->rg_tranloc->p_y);
@@ -942,16 +935,13 @@ ResGetTransistor(pt)
 	  tile = ResUse->cu_def->cd_planes[pnum]->pl_hint;
 	  GOTOPOINT(tile,&workingPoint);
 
-#ifdef NONMANHATTAN
 	  if (IsSplit(tile))
 	  {
               if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetLeftType(tile))
               	   || TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetRightType(tile)))
                   return(((tileJunk *)tile->ti_client)->transistorList);
 	  }
-	  else
-#endif
-          if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)))
+	  else if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)))
           {
                return(((tileJunk *)tile->ti_client)->transistorList);
           }
