@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selCreate.c,v 1.8 2010/05/11 19:28:54 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selCreate.c,v 1.10 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -474,7 +474,6 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	    bestMin, bestMax, bestChunk, level + 1);
 }
 
-#ifdef NONMANHATTAN
 /* Search function to find split tiles in the area of interest.
  */
 
@@ -496,8 +495,6 @@ selSplitFunc(tile, cxp)
     return 0;
 }
 
-#endif
-
 /* This procedure is called for each tile of the wrong type in an
  * area that is supposed to contain only tiles of other types.  It
  * just returns the area of the wrong material and aborts the search.
@@ -559,10 +556,7 @@ SelectChunk(scx, type, xMask, pArea, less)
     TileType ttype;
     Rect bestChunk;
     int bestMin, bestMax, width, height;
-
-#ifdef NONMANHATTAN
     extern int selSplitFunc();		/* Forward reference. */
-#endif
 
     /* If the source definition is changing, clear the old selection. */
 
@@ -593,7 +587,6 @@ SelectChunk(scx, type, xMask, pArea, less)
 
     newscx = *scx;
 
-#ifdef NONMANHATTAN
     /* If the tile under the area of interest is a split tile, 	*/
     /* then we use that tile as the chunk.  This routine should	*/
     /* work like the while() loop underneath, looking for the	*/
@@ -608,8 +601,6 @@ SelectChunk(scx, type, xMask, pArea, less)
         goto chunkdone;
     }
 
-#endif
-
     bestMin = bestMax = 0;
     bestChunk = GeoNullRect;
 
@@ -619,11 +610,7 @@ SelectChunk(scx, type, xMask, pArea, less)
 	/* Extract a bunch of junk. */
 
 	DBCellClearDef(Select2Def);
-#ifdef NONMANHATTAN
 	DBCellCopyManhattanPaint(&newscx, &typeMask, xMask, Select2Use);
-#else
-	DBCellCopyAllPaint(&newscx, &typeMask, xMask, Select2Use);
-#endif
 
 	/* Now find the best chunk in the area. */
 
@@ -961,6 +948,7 @@ SelectCell(use, rootDef, trans, replace)
 	(void) DBCellDeleteUse(selectLastUse);
 	SelRememberForUndo(FALSE, SelectRootDef, &area);
 	DBWHLRedraw(SelectRootDef, &area, TRUE);
+	selectLastUse = (CellUse *)NULL;
     }
 
     /* When creating a new use, try to re-use the id from the old
@@ -1129,7 +1117,8 @@ SelectAndCopy2(newSourceDef)
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selACPaintFunc,
 		(ClientData) plane);
-	DBMergeNMTiles(Select2Def->cd_planes[plane], &TiPlaneRect);
+	DBMergeNMTiles(Select2Def->cd_planes[plane], &TiPlaneRect,
+		(PaintUndoInfo *)NULL);
     }
 
     (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL,
@@ -1189,9 +1178,6 @@ selACPaintFunc(tile, plane)
     selACarg.editClip = &area;
     GeoTransRect(&RootToEditTransform, &area, &editArea);
 
-#ifndef NONMANHATTAN
-    selACarg.ttype = TiGetType(tile);
-#else
     selACarg.ttype = TiGetTypeExact(tile);
 
     if (IsSplit(tile))
@@ -1201,10 +1187,9 @@ selACPaintFunc(tile, plane)
 		selACPaintFunc2, (ClientData) &selACarg);
     }
     else
-#endif
-    (void) DBSrPaintArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[plane],
-	    &editArea, &DBAllButSpaceAndDRCBits, selACPaintFunc2,
-	    (ClientData) &selACarg);
+	(void) DBSrPaintArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[plane],
+		&editArea, &DBAllButSpaceAndDRCBits, selACPaintFunc2,
+		(ClientData) &selACarg);
     return 0;
 }
 
@@ -1229,16 +1214,11 @@ selACPaintFunc2(tile, selACarg)
     TileType type = TiGetTypeExact(tile);
     TileTypeBitMask tmask, *rmask;
     TileType ttype, rtype;
-
-#ifdef NONMANHATTAN
     TileType dinfo = selACarg->ttype & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE);
-#endif
 
     TiToRect(tile, &area);
     GeoTransRect(&EditToRootTransform, &area, &selArea);
 
-#ifdef NONMANHATTAN
-
     if ((dinfo & TT_DIAGONAL) || (type & TT_DIAGONAL))
     {
 	/* If the select area is triangular, then we need to	*/
@@ -1369,19 +1349,6 @@ topbottom:
 	}
 	TTMaskSetOnlyType(&tmask, rtype);
     }
-#else	/* !NONMANHATTAN */
-
-    ttype = selACarg->ttype;
-    if (type >= DBNumUserLayers)
-    {
-	rmask = DBResidueMask(type);
-	if (TTMaskHasType(rmask, ttype))
-	    type = ttype;
-    }
-    TTMaskSetOnlyType(&tmask, type);
-
-#endif
-
     GeoClip(&selArea, editClip);	/* editClip already in root coords */
 
 splitdone:
