@@ -564,13 +564,17 @@ proc magic::cursorview {win} {
    if {[catch {set cr [cif scale out]}]} {return}
    if {$cr == 0} {return}
    set olst [${win} cursor internal]
-   set olstx [expr [lindex $olst 0] * $cr]
-   set olsty [expr [lindex $olst 1] * $cr]
+
+   set olstx [expr [lindex $olst 0]]
+   set olsty [expr [lindex $olst 1]]
 
    if {$Opts(crosshair)} {
       *bypass crosshair ${olstx}i ${olsty}i
    }
 
+   set olstx [expr $olstx * $cr]
+   set olsty [expr $olsty * $cr]
+
    if {[${win} box exists]} {
       set dlst [${win} box position]
       set dx [expr {$olstx - ([lindex $dlst 0]) * $cr }]
@@ -610,6 +614,10 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
    if {$layerlist == "*"} {
       set layerlist [magic::tech layer "*"]
    }
+
+   # Change from comma-separated list to Tcl list
+   set layerlist [string map {, " "} $layerlist]
+
    foreach layer $layerlist {
       switch $layer {
 	 none {}
@@ -618,7 +626,7 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
 	 labels {set canon $layer}
 	 default {set canon [magic::tech layer $layer]}
       }
-      if {$layer != "none"} {
+      if {$layer != "none" && $canon != ""} {
          if {$yesno == "yes"} {
 	    ${framename}.toolbar.b$canon configure -image img_$canon 
 	    ${framename}.toolbar.p$canon configure -image pale_$canon 
@@ -729,7 +737,7 @@ proc magic::maketoolbar { framename } {
    # Figure out how many columns we need to fit all the layer buttons inside
    # the toolbar without going outside the window area.
 
-   set locklist [tech layer locked]
+   set locklist [tech locked]
    set ncols 1
    while {1} {
       incr ncols
@@ -742,12 +750,12 @@ proc magic::maketoolbar { framename } {
             grid ${framename}.toolbar.b$layername -row $i -column $j -sticky news
 	 }
 	 bind ${framename}.toolbar.p$layername <KeyPress-u> \
-		"$win tech layer unlock $layername ; \
+		"$win tech unlock $layername ; \
 		grid forget ${framename}.toolbar.p$layername ; \
 		grid ${framename}.toolbar.b$layername \
 		-row $i -column $j -sticky news"
 	 bind ${framename}.toolbar.b$layername <KeyPress-l> \
-		"$win tech layer lock $layername ; \
+		"$win tech lock $layername ; \
 		grid forget ${framename}.toolbar.b$layername ; \
 		grid ${framename}.toolbar.p$layername \
 		-row $i -column $j -sticky news"
@@ -1115,7 +1123,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    $m add separator
    $m add command -label "Print..."          -command { echo "not implemented" }
    $m add separator
-   $m add command -label "Save All and Quit" -command {magic::writeall force ; magic::quit -noprompt }
+   $m add command -label "Save All and Quit" -command {magic::writeall force ; if {[magic::cellname list modified] == {}} {magic::quit -noprompt} else {magic::quit} }
    $m add command -label "Quit"              -command {magic::quit}
 
 # #################################
@@ -1217,8 +1225,8 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 # Layers
 # #################################
    set m [menu ${framename}.titlebar.mbuttons.layers.toolmenu -tearoff 0]
-   $m add command -label "Protect Base Layers" -command {magic::tech layer revert  }
-   $m add command -label "Unlock  Base Layers" -command {magic::tech layer unlock *}
+   $m add command -label "Protect Base Layers" -command {magic::tech revert  }
+   $m add command -label "Unlock  Base Layers" -command {magic::tech unlock *}
    $m add separator
    $m add command -label "Clear Feedback"      -command {magic::feedback clear}
    $m add separator
