@@ -93,11 +93,7 @@ global char	*SysLibPath = NULL;	/* Used to find color maps, styles, */
  * (see utils.h for explanation of individual flags).
  */
 
-#ifdef MAGIC_WRAPPER
-global short RuntimeFlags = MAIN_MAKE_WINDOW | MAIN_TK_CONSOLE;
-#else
 global short RuntimeFlags = MAIN_MAKE_WINDOW;
-#endif
 
 /*
  * See the file main.h for a description of the information kept
@@ -176,6 +172,28 @@ MainExit(errNum)
     TxFlush();
     TxResetTerminal();
 
+#ifdef MAGIC_WRAPPER
+
+    // NOTE:  This needs to be done in conjunction with the following
+    // commands in the console:
+    // (1) tkcon eval rename ::exit ::quit
+    // (2) tkcon eval proc::exit args {slave eval quit}
+    //
+    // The lines above redirect tkcon's "exit" routine to be named
+    // "quit" (in the console, not the slave!).  Because the tkcon
+    // File->Exit callback is set to eval "exit", we then can create
+    // a new proc called "exit" in the console that runs "quit" in
+    // the slave (magic), and will therefore do the usual checks to
+    // save work before exiting;  if all responses are to exit without
+    // saving, then it finally gets to here, where it runs the (renamed)
+    // "quit" command in tkcon.  That will ensure that tkcon runs
+    // various cleanup activities such as saving the command-line
+    // history file before the final (!) exit.
+
+    if (TxTkConsole)
+	Tcl_Eval(magicinterp, "catch {tkcon eval quit}\n");
+#endif
+
     exit(errNum);
 }
 
@@ -368,12 +386,6 @@ mainDoArgs(argc, argv)
     }
 
     return 0;
-
-badfopt:
-    TxError("-F requires load and save file names, e.g.,");
-    TxError(" \"-F magic magicsave\"\n");
-exitnow:
-    return 1;
 }
 
 /*
@@ -450,10 +462,6 @@ mainInitBeforeArgs(argc, argv)
     LispInit();
 #endif
 
-    /* Identify version first thing */
-    TxPrintf("\nMagic %s revision %s - Compiled on %s.\n", MagicVersion,
-		MagicRevision, MagicCompileTime);
-
     /*
      * Get preliminary info on the graphics display.
      * This may be overriden later.
@@ -500,9 +508,6 @@ mainInitAfterArgs()
     if ((TechDefault == NULL) && (MainFileName != NULL))
 	(void) StrDup(&TechDefault, DBGetTech(MainFileName));
 
-    if (TechDefault == NULL)
-	TechDefault = "minimum";
-
     DBTypeInit();
     MacroInit();
 
@@ -643,34 +648,25 @@ mainInitAfterArgs()
 			sec_types, &sec_plot, TRUE);
 #endif
 
-    /* Load the technology file */
+    /* Load minimum technology file needed to keep things from	*/
+    /* crashing during initialization.				*/
 
-    if (!TechLoad(TechDefault, 0))
+    if (!TechLoad("minimum", 0))
     {
-	/* Don't give up just yet---user may have put a tech	*/
-	/* load command in the RC file.  Load the default	*/
-	/* technology, and only complain if that doesn't work	*/
-	/* either.						*/
-
-	if (!TechLoad("minimum", 0))
-	{
-	    TxError("Cannot load technology \"minimum\" or \"%s\"\n", TechDefault);
-	    return 2;
-	}
+	TxError("Cannot load technology \"minimum\" for initialization\n");
+	return 2;
     }
 
-#ifndef MAGIC_WRAPPER
+    /* The minimum tech has been loaded only to keep the database from	*/
+    /* becoming corrupted during initialization.  Free the tech file	*/
+    /* name so that a "real" technology file can be forced to replace	*/
+    /* it in mainInitFinal().						*/
 
-    // Let the wrapper script be responsible for formatting and
-    // printing the technology file informaiton.
-
-    if (DBTechName != 0) {
-	TxPrintf("Using technology \"%s\"", DBTechName);
-	if (DBTechVersion != 0) TxPrintf(", version %s.", DBTechVersion);
-	TxPrintf("\n");
+    if (TechFileName != NULL)
+    {
+	freeMagic(TechFileName);
+	TechFileName = NULL;
     }
-    if (DBTechDescription != 0) TxPrintf("%s\n", DBTechDescription);
-#endif
 
     /* initialize the undo package */
     (void) UndoInit((char *) NULL, (char *) NULL);
@@ -758,16 +754,71 @@ mainInitFinal()
     char *home, cwd[512];
     char startupFileName[256];
     FILE *f;
-
-    /* Read in system startup file, if it exists. */
-
-#ifdef MAGIC_WRAPPER
-
     char *rname;
     int result;
 
+#ifdef MAGIC_WRAPPER
+
+    /* Read in system pre-startup file, if it exists. */
+
     /* Use PaOpen first to perform variable substitutions, and	*/
-    /* return the actual filename in startupFileName.		*/
+    /* return the actual filename in rname.			*/
+
+    f = PaOpen(MAGIC_PRE_DOT, "r", (char *) NULL, ".",
+	    (char *) NULL, (char **) &rname);
+    if (f != NULL)
+    {
+	fclose(f);
+	result = Tcl_EvalFile(magicinterp, rname);
+	if (result != TCL_OK)
+	{
+	    TxError("%s\n", Tcl_GetStringResult(magicinterp));
+	    TxError("System pre-startup file \"%s\" not found or unreadable!\n",
+			rname);
+	    Tcl_ResetResult(magicinterp);
+	}
+    }
+#endif	/* MAGIC_WRAPPER */
+
+    /* Load the technology file.  If "site.pre" loaded a technology	*/
+    /* file, then "TechFileName" will be set, and we should not		*/
+    /* override it (but "site.pre" should not really be doing that).	*/
+
+    if ((TechFileName == NULL) && (TechDefault != NULL))
+    {
+	if (!TechLoad(TechDefault, -2))
+	    TxError("Failed to load technology \"%s\"\n", TechDefault);
+	else if (!TechLoad(TechDefault, 0))
+	    TxError("Error loading technology \"%s\"\n", TechDefault);
+    }
+
+    /* If that failed, then load the "minimum" technology again and	*/
+    /* keep it.  It's not very useful, but it will keep everything	*/
+    /* up and running.  In the worst case, if site.pre has removed the	*/
+    /* standard locations from the system path, then magic will exit.	*/
+
+    if (TechFileName == NULL)
+	if (!TechLoad("minimum", 0))
+	    return -1;
+
+#ifndef MAGIC_WRAPPER
+
+    // Let the wrapper script be responsible for formatting and
+    // printing the technology file informaiton.
+
+    if (DBTechName != 0) {
+	TxPrintf("Using technology \"%s\"", DBTechName);
+	if (DBTechVersion != 0) TxPrintf(", version %s.", DBTechVersion);
+	TxPrintf("\n");
+    }
+    if (DBTechDescription != 0) TxPrintf("%s\n", DBTechDescription);
+#endif
+
+#ifdef MAGIC_WRAPPER
+    /* Read in system startup file, if it exists. */
+
+    /* Use PaOpen first to perform variable substitutions, and	*/
+    /* return the actual filename in rname.			*/
 
     f = PaOpen(MAGIC_SYS_DOT, "r", (char *) NULL, ".",
 	    (char *) NULL, (char **) &rname);
